@@ -4,16 +4,19 @@
 import gobject, gtk
 import hildondesktop
 import hildon
-import subprocess
 import time
-from os import path
+import dbus
+from dbus.mainloop.glib import DBusGMainLoop
+from os import path, system
 from commands import getoutput
 from ConfigParser import SafeConfigParser
 
+
 class opensshStatus(hildondesktop.StatusMenuItem):
     def __init__(self):
         hildondesktop.StatusMenuItem.__init__(self)
 
+        self.trackingEnabled = True
         self.stopTracking = True
         self.stopWhenIdle = False
         self.stopAfterMinutes = 5
@@ -28,13 +31,15 @@ class opensshStatus(hildondesktop.StatusMenuItem):
             try:
                 parser.read('/home/user/.openssh-status-widget')
             except:
-                subprocess.call(["rm", "/home/user/.openssh-status-widget"])
+                system("rm /home/user/.openssh-status-widget")
+            if parser.has_option('options', 'trackingEnabled'):
+                self.trackingEnabled = parser.getboolean('options', 'trackingEnabled')
             if parser.has_option('options', 'stopTracking'):
                 self.stopTracking = parser.getboolean('options', 'stopTracking')
             if parser.has_option('options', 'stopWhenIdle'):
                 self.stopWhenIdle = parser.getboolean('options', 'stopWhenIdle')
             if parser.has_option('options', 'stopAfterMinutes'):
-                self.stopAfterMinutes = int(parser.get('options', 'stopAfterMinutes').strip('_'))
+                self.stopAfterMinutes = int(parser.get('options', 'stopAfterMinutes').split()[0])
             if parser.has_option('options', 'keepWifiPMOff'):
                 self.keepWifiPMOff = parser.getboolean('options', 'keepWifiPMOff')
             if parser.has_option('options', 'confirmDisconnect'):
@@ -42,13 +47,13 @@ class opensshStatus(hildondesktop.StatusMenuItem):
             if parser.has_option('options', 'showConnected'):
                 self.showConnected = parser.getboolean('options', 'showConnected')
             if parser.has_option('options', 'connectedIcon'):
-                connectedIcon = parser.get('options', 'connectedIcon')
+                connectedIcon = parser.get('options', 'connectedIcon').strip('_')
             if parser.has_option('options', 'skipSystemCheck'):
                 self.skipSystemCheck = parser.getboolean('options', 'skipSystemCheck')
             
         self.icon=gtk.gdk.pixbuf_new_from_file("/usr/share/icons/hicolor/18x18/hildon/openssh-status_no-users.png")
         try:
-            self.icon_plus=gtk.gdk.pixbuf_new_from_file("/usr/share/icons/hicolor/18x18/hildon/openssh-status_users_"+connectedIcon.lower().strip('_')+".png")
+            self.icon_plus=gtk.gdk.pixbuf_new_from_file("/usr/share/icons/hicolor/18x18/hildon/openssh-status_users_"+connectedIcon.lower()+".png")
         except:
             self.icon_plus=gtk.gdk.pixbuf_new_from_file("/usr/share/icons/hicolor/18x18/hildon/openssh-status_users_symbol.png")
 
@@ -68,14 +73,36 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         self.waitWhileIdle = False
         self.refresh = False
         self.refreshWifiPMOff = False
+        self.loops = 1
         self.labelConnected = 0
         self.sshJustOff = 0
         
+
         self.start()
 
         self.button.connect("clicked", self.buttonClicked)
         self.button.tap_and_hold_setup(None, None, 0)
         self.button.connect("tap-and-hold", self.buttonPressed)
+
+        DBusGMainLoop(set_as_default=True)
+        bus = dbus.SystemBus()
+        bus.add_signal_receiver(self.handleDisplay, path='/com/nokia/mce/signal', dbus_interface='com.nokia.mce.signal', signal_name='display_status_ind')
+        mceObj = bus.get_object("com.nokia.mce", "/com/nokia/mce/request")
+        mceIF = dbus.Interface(mceObj, "com.nokia.mce.request")
+        displayStatus = mceIF.get_display_status()
+        self.handleDisplay(displayStatus)
+        
+    def handleDisplay(self, displayStatus):
+        if displayStatus == "on" or displayStatus == "dimmed":
+            self.loops = 0
+            if self.refresh:
+                self.sshTracker()
+        if displayStatus == "off":
+            self.loops = 17279
+            if self.stopWhenIdle:
+                self.loops = 11
+                if self.stopAfterMinutes == 1:
+                    self.loops = 5
         
     def start(self):
         if not self.skipSystemCheck:
@@ -91,35 +118,48 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                 self.adjustSshFile()
         
         self.sshStatus()
-        self.sshUsers()
+        if self.trackingEnabled:
+            self.sshUsers()
         if self.sshRunning:
             self.on = True
             self.button.set_image(self.image_on)
-            if self.sshConnected and self.showConnected:
-                self.connected = True
-                self.set_status_area_icon(self.icon_plus)
-            else:
-                self.set_status_area_icon(self.icon)
-            self.refresh = True
-            if not self.sshConnected:
-                self.button.set_text("OpenSSH Server", "0 connections")
-            gobject.timeout_add(5000, self.sshTracker)
-        else:
-            self.button.set_image(self.image_off)
-            if self.sshConnected:
-                if self.showConnected:
+            if self.trackingEnabled:
+                if self.sshConnected and self.showConnected:
                     self.connected = True
                     self.set_status_area_icon(self.icon_plus)
                 else:
                     self.set_status_area_icon(self.icon)
                 self.refresh = True
+                if not self.sshConnected:
+                    self.button.set_text("OpenSSH Server", "0 connections")
+                self.loopCount = -1
+                self.sshTracker()
                 gobject.timeout_add(5000, self.sshTracker)
+            else:
+                self.set_status_area_icon(self.icon)
+                self.button.set_text("OpenSSH Server", "On")
+        else:
+            self.button.set_image(self.image_off)
+            if self.trackingEnabled:
+                if self.sshConnected:
+                    if self.showConnected:
+                        self.connected = True
+                        self.set_status_area_icon(self.icon_plus)
+                    else:
+                        self.set_status_area_icon(self.icon)
+                    self.refresh = True
+                    self.loopCount = -1
+                    gobject.timeout_add(5000, self.sshTracker)
+                else:
+                    self.set_status_area_icon(None)
+                    self.button.set_text("OpenSSH Server", "Off")
+                    if not self.stopTracking:
+                        self.refresh = True
+                        self.loopCount = -1
+                        gobject.timeout_add(5000, self.sshTracker)
             else:
                 self.set_status_area_icon(None)
                 self.button.set_text("OpenSSH Server", "Off")
-                if not self.stopTracking:
-                    self.refresh = True
-                    gobject.timeout_add(5000, self.sshTracker)
             
     def adjustSshFile(self):
         self.dialog = gtk.Dialog("OpenSSH status", None)
@@ -138,7 +178,7 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         self.dialog.show_all()
         
     def adjustYes(self, dummy):
-        subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "disable_respawn"])
+        system("sudo /usr/sbin/openssh-status.sh disable_respawn")
         self.dialog.destroy()
         message = gtk.Dialog("OpenSSH status", None)
         text = gtk.Label("Done! Please reboot the N900.")
@@ -171,88 +211,92 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         self.sshConnected = int(getoutput("ps aux | grep sshd: | grep -c @")) - 1
         
     def sshTracker(self):
-        self.sshRunning = int(getoutput("ps aux | grep -c /usr/sbin/sshd")) - 2
-        self.sshConnected = int(getoutput("ps aux | grep sshd: | grep -c @")) - 1
-        if self.sshRunning and self.sshJustOff:
-            self.sshJustOff = 0
-        if not self.sshRunning and self.on:
-            self.on = False
-            self.button.set_image(self.image_off)
-        if self.sshRunning and not self.on:
-            self.on = True
-            self.button.set_image(self.image_on)
-            if self.sshConnected and self.showConnected:
-                self.connected = True
-                self.set_status_area_icon(self.icon_plus)
-            else:
-                self.connected = False
-                self.set_status_area_icon(self.icon)
-        if self.showConnected:
-            if self.sshConnected and not self.connected:
-                self.connected = True
-                self.set_status_area_icon(self.icon_plus)
-            if self.sshRunning and not self.sshConnected and self.connected:
-                self.connected = False
-                self.set_status_area_icon(self.icon)
-        if self.sshConnected and self.keepWifiPMOff:
-            if not self.refreshWifiPMOff:
-                self.refreshWifiPMOff = True
-            subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "disable_wifipm"])
-        elif self.refreshWifiPMOff:
-            self.refreshWifiPMOff = False
-            subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "enable_wifipm"])
-        if self.sshConnected != self.labelConnected:
-            self.labelConnected = self.sshConnected
-            self.waitWhileIdle = False
-            if self.sshConnected == 1:
-                self.button.set_text("OpenSSH Server", str(self.sshConnected)+" connection")
-                return True
-            elif self.sshConnected > 1 or (not self.sshConnected and self.sshRunning):
-                self.button.set_text("OpenSSH Server", str(self.sshConnected)+" connections")
-                return True
-        if not self.sshConnected and not self.sshRunning:
-            #if self.sshJustOff < 1:
-            #    self.sshJustOff += 1
-            #    return True
-            #else:
-            self.on = False
-            self.button.set_image(self.image_off)
-            self.set_status_area_icon(None)
-            self.button.set_text("OpenSSH Server", "Off")
-            if self.stopTracking:
-                if self.sshJustOff < 3:
-                    self.sshJustOff += 1
+        # Smart frequency
+        self.loopCount += 1
+        if self.loopCount > self.loops:
+            self.loopCount = 0
+        # Tracker
+        if not self.loopCount or self.sshJustOff:
+            self.sshStatus()
+            self.sshUsers()
+            if self.sshRunning and self.sshJustOff:
+                self.sshJustOff = 0
+            if not self.sshRunning and self.on:
+                self.on = False
+                self.button.set_image(self.image_off)
+            if self.sshRunning and not self.on:
+                self.on = True
+                self.button.set_image(self.image_on)
+                if self.sshConnected and self.showConnected:
+                    self.connected = True
+                    self.set_status_area_icon(self.icon_plus)
+                else:
+                    self.connected = False
+                    self.set_status_area_icon(self.icon)
+            # Update Status area icon
+            if self.showConnected:
+                if self.sshConnected and not self.connected:
+                    self.connected = True
+                    self.set_status_area_icon(self.icon_plus)
+                if self.sshRunning and not self.sshConnected and self.connected:
+                    self.connected = False
+                    self.set_status_area_icon(self.icon)
+            # Update button
+            if self.sshConnected != self.labelConnected:
+                self.labelConnected = self.sshConnected
+                self.waitWhileIdle = False
+                if self.sshConnected == 1:
+                    self.button.set_text("OpenSSH Server", str(self.sshConnected)+" connection")
                     return True
+                elif self.sshConnected > 1 or (not self.sshConnected and self.sshRunning):
+                    self.button.set_text("OpenSSH Server", str(self.sshConnected)+" connections")
+                    return True
+            # Stop tracking when server is off
+            if not self.sshConnected and not self.sshRunning:
+                self.on = False
+                self.button.set_image(self.image_off)
+                self.set_status_area_icon(None)
+                self.button.set_text("OpenSSH Server", "Off")
+                if self.stopTracking:
+                    if self.sshJustOff < 3:
+                        self.sshJustOff += 1
+                        return True
+                    else:
+                        self.sshJustOff = 0
+                        self.refresh = False
+                        self.waitWhileIdle = False
+                        if self.keepWifiPMOff:
+                            self.refreshWifiPMOff = False
+                            system("sudo /usr/sbin/openssh-status.sh enable_wifipm")
+                            if self.wifiTX10:
+                                system("sudo /usr/sbin/openssh-status.sh switch_wifitx")
+                        return False
                 else:
-                    self.sshJustOff = 0
-                    self.refresh = False
-                    self.waitWhileIdle = False
-                    return False
-            else:
-                return True
-        if not self.sshConnected and self.stopWhenIdle:
-            if not self.waitWhileIdle:
-                self.idleStartTime = time.time()
-                self.waitWhileIdle = True
-                return True
-            elif time.time() - self.idleStartTime + 5 > self.stopAfterMinutes * 60:
-                subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "stop_sshd"])
-                time.sleep(1)
-                self.sshStatus()
-                if self.sshRunning:
-                    self.printDialog("Could NOT stop OpenSSH Server!")
-                else:
-                    self.on = False
-                    self.waitWhileIdle = False
-                    self.button.set_text("OpenSSH Server", "Off")
-                    self.button.set_image(self.image_off)
-                    self.set_status_area_icon(None)
+                    return True
+            # Stop server when not connected
+            if not self.sshConnected and self.stopWhenIdle:
+                if not self.waitWhileIdle:
+                    self.idleStartTime = time.time()
+                    self.waitWhileIdle = True
+                    return True
+                elif time.time() - self.idleStartTime + 5 > self.stopAfterMinutes * 60:
+                    system("sudo /usr/sbin/openssh-status.sh stop_sshd")
+                    time.sleep(1)
+                    self.sshStatus()
+                    if self.sshRunning:
+                        self.printDialog("Could NOT stop OpenSSH Server!")
+                    else:
+                        self.on = False
+                        self.waitWhileIdle = False
+                        self.button.set_text("OpenSSH Server", "Off")
+                        self.button.set_image(self.image_off)
+                        self.set_status_area_icon(None)
         return True
         
     def buttonClicked(self, dummy):
         self.sshStatus()
         if self.sshRunning and self.on:
-            subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "stop_sshd"])
+            system("sudo /usr/sbin/openssh-status.sh stop_sshd")
             time.sleep(1)
             self.sshStatus()
             if self.sshRunning:
@@ -260,45 +304,88 @@ class opensshStatus(hildondesktop.StatusMenuItem):
             else:
                 self.on = False
                 self.button.set_image(self.image_off)
-                self.sshUsers()
-                if not self.sshConnected:
+                if self.trackingEnabled:
+                    self.sshUsers()
+                    if not self.sshConnected:
+                        self.set_status_area_icon(None)
+                        self.button.set_text("OpenSSH Server", "Off")
+                else:
                     self.set_status_area_icon(None)
                     self.button.set_text("OpenSSH Server", "Off")
+                    
         elif self.sshRunning and not self.on:
             self.on = True
             self.button.set_image(self.image_on)
-            self.sshUsers()
-            if self.sshConnected:
-                self.button.set_text("OpenSSH Server", "")
-                if self.showConnected:
-                    self.connected = True
-                    self.set_status_area_icon(self.icon_plus)
+            if self.trackingEnabled:
+                self.sshUsers()
+                if self.sshConnected:
+                    if self.sshConnected == 1:
+                        self.button.set_text("OpenSSH Server", "1 connection")
+                    else:
+                        self.button.set_text("OpenSSH Server", str(self.sshConnected)+" connections")
+                    if self.showConnected:
+                        self.connected = True
+                        self.set_status_area_icon(self.icon_plus)
+                    else:
+                        self.connected = False
+                        self.set_status_area_icon(self.icon)
                 else:
-                    self.connected = False
-                    self.set_status_area_icon(self.icon)
-            else:
-                self.button.set_text("OpenSSH Server", "0 connections")
-                self.connected = False
-                self.set_status_area_icon(self.icon)
-            if not self.refresh:
-                self.refresh = True
-                gobject.timeout_add(5000, self.sshTracker)
-            self.printDialog("OpenSSH Server already running! Display updated.")
-        else:
-            subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "start_sshd"])
-            time.sleep(1)
-            self.sshStatus()
-            self.sshUsers()
-            if self.sshRunning:
-                self.on = True
-                self.button.set_image(self.image_on)
-                if not self.sshConnected:
                     self.button.set_text("OpenSSH Server", "0 connections")
                     self.connected = False
                     self.set_status_area_icon(self.icon)
                 if not self.refresh:
                     self.refresh = True
+                    self.loopCount = -1
                     gobject.timeout_add(5000, self.sshTracker)
+            else:
+                self.set_status_area_icon(self.icon)
+                self.button.set_text("OpenSSH Server", "On")
+            self.printDialog("OpenSSH Server already running! Display updated.")
+        elif not self.sshRunning and self.on:
+            self.on = False
+            self.button.set_image(self.image_off)
+            if self.trackingEnabled:
+                self.sshUsers()
+                if self.sshConnected:
+                    if self.sshConnected == 1:
+                        self.button.set_text("OpenSSH Server", "1 connection")
+                    else:
+                        self.button.set_text("OpenSSH Server", str(self.sshConnected)+" connections")
+                    if self.showConnected:
+                        self.connected = True
+                        self.set_status_area_icon(self.icon_plus)
+                    else:
+                        self.connected = False
+                        self.set_status_area_icon(self.icon)
+                else:
+                    self.button.set_text("OpenSSH Server", "Off")
+                    self.connected = False
+                    self.set_status_area_icon(None)
+                self.refresh = False
+            else:
+                self.set_status_area_icon(None)
+                self.button.set_text("OpenSSH Server", "Off")
+            self.printDialog("OpenSSH Server already stopped! Display updated.")
+        else:
+            system("sudo /usr/sbin/openssh-status.sh start_sshd")
+            time.sleep(1)
+            self.sshStatus()
+            if self.sshRunning:
+                self.on = True
+                self.button.set_image(self.image_on)
+                if self.trackingEnabled:
+                    self.sshUsers()
+                    if not self.sshConnected:
+                        self.button.set_text("OpenSSH Server", "0 connections")
+                        self.connected = False
+                        self.set_status_area_icon(self.icon)
+                    if not self.refresh:
+                        self.refresh = True
+                        self.loopCount = -1
+                        gobject.timeout_add(5000, self.sshTracker)
+                else:
+                    self.set_status_area_icon(self.icon)
+                    self.button.set_text("OpenSSH Server", "On")
             else:
                 self.printDialog("Could NOT start OpenSSH Server!")
 
@@ -306,32 +393,53 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         self.button.get_toplevel().hide()
         self.sshStatus()
         self.sshUsers()
-        if (self.sshRunning or self.sshConnected) and self.confirmDisconnect:
+        if (self.sshRunning or self.sshConnected) and (self.confirmDisconnect or not self.trackingEnabled):
             self.dialog = gtk.Dialog("OpenSSH status", None)
             text = gtk.Label("Disconnect and stop server?\n")
             text.set_line_wrap(True)
-            self.dialog.vbox.pack_start(text)
-            self.dialog.add_button("Disconnect", gtk.RESPONSE_YES)
-            self.dialog.add_button("Abort", gtk.RESPONSE_NO)
+            buttonDisconnectStop = hildon.Button(gtk.HILDON_SIZE_THUMB_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+            buttonDisconnectStop.set_label('Disconnect and stop server')
+            if not self.trackingEnabled:
+                buttonConnected = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+                buttonConnected.set_label('Connections')
+                if self.sshConnected == 1:
+                    self.printInfo("1 connection")
+                else:
+                    self.printInfo(str(self.sshConnected)+" connections")
+            vbox2 = gtk.VBox(False,2)
+            vbox2.set_border_width(8)
+            if not self.trackingEnabled:
+                vbox2.pack_start(buttonConnected)
+                buttonConnected.connect("clicked", self.buttonConnected_clicked)
+            vbox2.pack_start(buttonDisconnectStop)
+            buttonDisconnectStop.connect("clicked", self.buttonDisconnectStop_clicked)
+            self.dialog.vbox.pack_start(vbox2)
             self.dialog.show_all()
-            response = self.dialog.run()
-            if response <> gtk.RESPONSE_YES:
-                self.dialog.destroy()
-                return
-            else:
-                self.dialog.destroy()
-                self.sshDisconnect() 
         elif self.sshRunning or self.sshConnected:
             self.sshDisconnect()
         else:
-            time.sleep(1)
-            self.printDialog("OpenSSH Server already stopped!")
+            if self.trackingEnabled:
+                time.sleep(1)
+                self.printInfo("OpenSSH Server already stopped!")
+            else:
+                self.buttonConnected_clicked(0)
         
+    def buttonDisconnectStop_clicked(self, dummy):
+        self.dialog.destroy()
+        self.sshDisconnect()
+    
+    def buttonConnected_clicked(self, dummy):
+        self.sshUsers()
+        if self.sshConnected == 1:
+            self.printInfo("1 connection")
+        else:
+            self.printInfo(str(self.sshConnected)+" connections")
+    
     def sshDisconnect(self):
         self.sshStatus()
         self.sshUsers()
         if self.sshRunning:
-            subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "kill_sshd"])
+            system("sudo /usr/sbin/openssh-status.sh kill_sshd")
             time.sleep(1)
             self.sshStatus()
             self.sshUsers()
@@ -345,7 +453,7 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                 self.button.set_image(self.image_off)
                 self.set_status_area_icon(None)
         elif self.sshConnected:
-            subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "kill_sshd"])
+            system("sudo /usr/sbin/openssh-status.sh kill_sshd")
             time.sleep(1)
             self.sshUsers()
             if self.sshConnected:
@@ -355,14 +463,17 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                 self.button.set_text("OpenSSH Server", "Off")
                 self.button.set_image(self.image_off)
                 self.set_status_area_icon(None)
-        else:
-            time.sleep(1)
-            self.printDialog("OpenSSH Server already stopped!")
             
     def printDialog(self, msg):
-        subprocess.call(["run-standalone.sh", "dbus-send", "--print-reply", \
-        "--type=method_call", "--dest=org.freedesktop.Notifications", \
-        "/org/freedesktop/Notifications", "org.freedesktop.Notifications.SystemNoteDialog", \
-        "string:"+msg, "uint32:0", "string:"])
+        system("run-standalone.sh dbus-send --print-reply \
+        --type=method_call --dest=org.freedesktop.Notifications \
+        /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteDialog \
+        string:\""+msg+"\" uint32:0 string:")
+
+    def printInfo(self, msg):
+        system("run-standalone.sh dbus-send --print-reply \
+        --type=method_call --dest=org.freedesktop.Notifications \
+        /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint \
+        string:\""+msg+"\"")
 
 hd_plugin_type = opensshStatus
